#!/bin/bash
#          _      _   _        ___       _    _  _
#  ___ ___| |_  _| |_(_)___   / __|_ __ | |__| || |
# (_-</ _ \ | || |  _| / _ \ | (_ | '  \| '_ \ __ |
# /__/\___/_|\_,_|\__|_\___/  \___|_|_|_|_.__/_||_|


echo "received raw parameters: $@"


# well known places for Java (8)
ADOPTIUM_JRE="/Library/Java/JavaVirtualMachines/temurin-8.jre/Contents/Home/bin/java"
ADOPTIUM_JDK="/Library/Java/JavaVirtualMachines/temurin-8.jdk/Contents/Home/jre/bin/java"
 
ADOPT_JRE="/Library/Java/JavaVirtualMachines/adoptopenjdk-8.jre/Contents/Home/bin/java"
ADOPT_JDK="/Library/Java/JavaVirtualMachines/adoptopenjdk-8.jdk/Contents/Home/jre/bin/java"

ORACLE_JAVA="/Library/Internet Plug-Ins/JavaAppletPlugin.plugin/Contents/Home/bin/java"

ADOPTIUM_DOWNLOAD_LINK="https://adoptium.net/?variant=openjdk8&jvmVariant=hotspot"	

JAVA_CMD=""

# Find the default Java 8 installation as known to MacOS
findDefaultJava() {
  echo -n "Lookup default java installation... "
  JAVA_HOME_OUT=$( /usr/libexec/java_home -v 1.8 2>&1 )
  JAVA_HOME_RC=$?
  DEFAULT_JAVA_HOME=$( echo $JAVA_HOME_OUT | head -n 1 )
  echo $DEFAULT_JAVA_HOME
  if [[ "$JAVA_HOME_RC" = "0" ]]; then
    JAVA_CMD="${DEFAULT_JAVA_HOME}/bin/java"
  else
    echo "Let the user install a fresh Java version"
  fi
}

# Find JRE-installations in well-known places
# with fallback to findDefaultJava
findJRE() {
  if [[ -x $ADOPTIUM_JRE ]]; then
    echo -n "Found Adoptium JRE... "
    JAVA_CMD=$ADOPTIUM_JRE
  elif [[ -x $ADOPTIUM_JDK ]]; then
    echo -n "Found Adoptium JDK... "
    JAVA_CMD=$ADOPTIUM_JDK
  elif [[ -x $ADOPT_JRE ]]; then
    echo -n "Found AdoptOpenJDK JRE... "
    JAVA_CMD=$ADOPT_JRE
  elif [[ -x $ADOPT_JDK ]]; then
    echo -n "Found AdoptOpenJDK JDK... "
    JAVA_CMD=$ADOPT_JDK
  elif [[ -x $ORACLE_JAVA ]]; then
    echo -n "Found Oracle JRE... "
    JAVA_CMD=$ORACLE_JAVA
  else
    echo "Neither Adoptium nor AdoptOpenJDK nor Oracle JRE found... "
    findDefaultJava
  fi
}

# first determine the newest JRE (an Adopt JRE 1.8)
findJRE

echo "Java command: ${JAVA_CMD}"

echo -n "check java version: "


JAVA_VERSION="N/A"
if [[ -n "$JAVA_CMD" ]]
then
	JAVA_VERSION=$("$JAVA_CMD" -version 2>&1 | head -n 1 | cut -d\" -f 2)
fi

echo $JAVA_VERSION

if [[ "$JAVA_VERSION" == "N/A" ]] || [[ "$JAVA_VERSION" = 1.3.* ]] || [[ "$JAVA_VERSION" = 1.4.* ]] || [[ "$JAVA_VERSION" = 1.5.* ]] || [[ "$JAVA_VERSION" = 1.6.* ]] || [[ "$JAVA_VERSION" = 1.7.* ]]
then
	osascript -e 'tell application "System Events" to display dialog "Es wurde keine kompatible Java Version gefunden. Es ist mindestens Java in der folgenden Version erforderlich: 1.8.0" buttons {"OK"} default button 1 with title "charly-Updater" with icon caution'
	open $ADOPTIUM_DOWNLOAD_LINK
	exit 0
fi


echo "setup working directory"

# determine actual directory
SOURCE="${BASH_SOURCE[0]}"
DIR="$( dirname "$SOURCE" )"
while [ -h "$SOURCE" ]
do
  SOURCE="$(readlink "$SOURCE")"
  [[ $SOURCE != /* ]] && SOURCE="$DIR/$SOURCE"
  DIR="$( cd -P "$( dirname "$SOURCE"  )" && pwd )"
done
DIR="$( cd -P "$( dirname "$SOURCE" )" && pwd )"

cd "$DIR"


echo "arguments received: " $@

# patch given arguments: -suite, qs_charly_updatestable, -suiteversion, 1337, -psn_0_6932124
# the parameter -psn_*** is not wanted!!!

PARAMLIST=""
for PARAM in "$@"; do
  if [[ "${PARAM:0:4}" != "-psn" ]]; then
  	PARAMLIST+=" "
    PARAMLIST+="'$PARAM'"
  fi
done

echo "arguments after patch: " $PARAMLIST

PARENT_DIR=${DIR%/*}
LAUNCHER_PATH=${PARENT_DIR%/*}
LOG_DIR=$(eval echo ~)

# determine effective userinformation
EFFECTIVE_USER=${SUDO_USER:-$USER}
EFFECTIVE_USERHOME=$(eval echo ~${EFFECTIVE_USER})


# change to parent directory of application bundle to be compatible with windows working directory
cd "$LAUNCHER_PATH/.."

# launch the application
"${JAVA_CMD}" -Xdock:name="charly-Updater" -Xms128m -Xmx512m -Xdock:icon="$DIR/../Resources/charly_v2.icns" -Dfile.encoding="UTF-8" -Dlog.dir="/Library/Application Support/solutio/updater/meta" -Djava.awt.headless=false -Dlaunch.appfile="$LAUNCHER_PATH" -Dupdaterupdate="$PARAMLIST" -jar "$LAUNCHER_PATH/Contents/Resources/java/updater-application.jar" -adminusr "${EFFECTIVE_USER}"


RETURN_VALUE=$?

if [[ "$RETURN_VALUE" != "0" ]]; then
	echo "charly-Updater failed with exit code $RETURN_VALUE"
	if [ "$RETURN_VALUE" = "126" ]; then
		echo "charly-Updater is not executable"
	elif [ "$RETURN_VALUE" = "127" ]; then
		echo "charly-Updater requires Java 1.8"
		open $ADOPTIUM_DOWNLOAD_LINK
	elif [ "$RETURN_VALUE" = "130" ]; then
		echo "charly-Updater terminated by Control-C"
	else
		echo "charly-Updater error"
	fi
fi

exit $RETURN_VALUE
